{************************************************
 *                                              *
 *    SAT-Master v1.75  Hauptprogramm SAT.PAS   *
 *                                              *
 ************************************************

procedure trackmap;
procedure scanner;
procedure datamenu;
procedure illumination(dtime:extended);
function eklipse:boolean;
procedure info;
procedure bahnberechnung;
procedure bahnhoehe;
procedure illprognose;
procedure eklprognose;
procedure mainwin;
**********************************}
{$A+,L-,R-,S-,I+,F-,O-,N+,E+,L-,V+,B-,M 16384 0 655360}
program satellitenbahnberechnung; (* TP 5.5/6.0 *)

uses   antenne, crt, dos, graph, graph2, mathe;

const  version : string[4] = '1.75';
       copyright : string[10] = 'DB1HZ 1991';
       maxsat  : byte      = 253;                {max. Satellitenzahl}
       bm      : extended  = 0.0174532925199327; {pi/180}
       fp      : extended  = 100.2907;           {Fruehlingspunkt 1978}
       zf      : extended  = 360.985647332099;   {Erdrotation in Grad pro Tag}
                                {gegen Himmel - ein Tag dauert 23h 56min 4sek}
       er      : extended  = 6378.145;           {mittlerer Erdradius in km}
       ae      : extended  = 149500000;          {Astronomische Einheit in km}
       crlf    : string[2] = chr(10)+chr(13);

var    ekl, mekl                                      : boolean;
       sat, wahl, manomalie                           : byte;
       sek100, sekunde, minute, stunde, tag, wtag, monat, jahr,
       mminute, mtag, dummy, banf, bend               : word;
       j, x, y, xmax, ymax, zeitzone, ssph, sspv,
       sspi, sspj, mazimut, melevation, squint,
       msquint, midx, midy                            : integer;
       entfernung, mentfernung, sspr, msspr,
       orbit, morbit                                  : longint;
       sspl, msspl, sspb, msspb, laenge, breite, sil, rechnerzeit,
       sspe, sib, sir, tref, pi2, anomalie, azimut,
       elevation, ill, alphaSo, deltaSo, GrtWi, GrtSo : extended;
       key                                            : char;
       txt, textzeile                                 : string[100];
       memssp                                         : pointer;
       satet, satincl, satraan, satecc, satargp, satma,
       satmm, satemm, satda, sataa       : array[1..253] of extended;
       satuml                            : array[1..253] of longint;
       satnam                            : array[1..253] of string[30];


{$I SAT1.INC}

{$I SAT2.INC}

procedure trackmap;
{*** zeichnet die Spurkarte ***}
var min, x1, y1, x2, y2, ej : integer;
    f : byte;
    tbasis,dt,et,zdx,zdy,zdz,zmm,za,zb,zc,zd,zg,zh,zj,zk,zl,zm,zn,
    zo,zp,zq,zr,zs,zt,zu,zv,zw,zx,zy,zz : extended;

  procedure schleife(var zp,zr,zx,zy:extended);
  begin
    zr:=sqrt(zx*zx+zy*zy);
    if zx=0 then if zy<0 then zp:=-90 else zp:=90
    else begin
      zp:=arctan(zy/zx)/bm;
      if zx<=0 then if zy>0 then zp:=zp+180 else zp:=zp-180
    end
  end;

begin
  if wahl>253 then fehlt else begin
    sspi:=0;
    sspj:=0;
    mtag:=0;
    mminute:=61;
    min:=0;
    dt:=1/(1024*satmm[wahl]);
    grwritexy(8,1,'Uhr',14);
    grwritexy(36,1,satnam[wahl],14);
    grwritexy(71,1,copyright,15);
    zeileoben;
    txt:='Spurkarte der nchsten Umlufe';
    grwritexy(26,uz,txt,14);
    setpalette(9,63);
    setpalette(10,63);
    setpalette(12,63);
    zdz:=breite*bm;
    systime;
    zt:=time;                     {aktuelle Zeit in Tagen absolut}
    zg:=amsattime(wahl);          {Zeit in Tagen des Referenzperigaeums absolut}
    repeat
      zd:=zt-zg;                            {Tage seit Referenzperigaeum}
      zmm:=satmm[wahl]+satemm[wahl]*zd;     {akt. Mean Motion}
      zn:=pi2*zmm;                          {Winkelgeschw.}
      za:=42220*hoch(zmm,-2/3)/er;    {gr. Bahnhalbachse nach Kepler: U^2=T^3}
      zz:=hoch(za,3.5)*sqr(1-satecc[wahl]*satecc[wahl]);
      zv:=cos(satincl[wahl]*bm);
      zq:=-9.9999999999*zv/zz;              {dArgPer pro Tag}
      zv:=(25*zv*zv-5)/zz;                  {dRAAN pro Tag}
      zo:=satraan[wahl]+zd*zq;              {akt. ArgPer}
      zw:=satargp[wahl]+zd*zv;              {akt. RAAN}
      zk:=zo-fp-zt*zf-laenge;               {Laengendifferenz}
      zm:=zd*zn;                            {Differenzumlaeufe}
      zr:=satecc[wahl];                     {Naeherung nach Kepler}
      zp:=zm;                               {im Bogenma}
      repeat                                {Algorithmus nach Dr.Karl Meinzer}
        zh:=(zm-zp+zr*sin(zp))/(1-zr*cos(zp));           {AMSAT-DL/CQ-DL 1985}
        zp:=zp+zh;
      until abs(zh)<1E-10;
      zy:=sin(zp)*sqrt(1-satecc[wahl]*satecc[wahl]);
      zx:=cos(zp)-satecc[wahl];
      schleife(zp,zr,zx,zy);
      zr:=za*zr;
      zp:=bm*(zp+zw);
      zy:=zr*sin(zp);
      zh:=zr*cos(zp);
      zp:=satincl[wahl]*bm;
      zs:=zy*sin(zp);
      zx:=zy*cos(zp);
      schleife(zp,zr,zh,zx);
      zp:=zp+zk;
      zk:=zp+laenge;
      zu:=zr;
      zp:=zp*bm;
      zh:=zr*sin(zp);
      zz:=zs-sin(zdz);
      zx:=zr*cos(zp)-cos(zdz);
      schleife(zp,zr,zx,zz);
      zp:=bm*(zp-breite+90);
      zj:=zr*sin(zp);
      zx:=zr*cos(zp);
      schleife(zp,zr,zx,zh);
      zc:=180-zp;
      zx:=zr;
      schleife(zp,zr,zx,zj);               {soweit Fertig}
      zk:=360*frac(zk/360);                {SubSatellitenPunkt berechnen}
      schleife(zp,zr,zu,zs);
      if zk<-180 then zk:=zk+360;          {Anpassen fr Ausgabe}
      if zk>180 then zk:=zk-360;
      x1:=round((zk+180)*x/360);
      y1:=round(y-y/180*(zp+90)+offset);
      f:=getpixel(x1,y1);
      if f<8 then putpixel(x1,y1,f+8);
      zt:=zt+dt;
      inc(min);
      if frac(min/512)=0 then begin
        systime;
        zeileoben;
        min:=0
      end;
    until keypressed;
    while keypressed do key:=readkey;
    repeat
      systime;
      zeileoben;
    until keypressed;
    fastblock(offset,y+offset);
    setpalette(9,57);
    setpalette(10,58);
    setpalette(12,60);
    dellines;
    waitkey
  end;
  key:='@'
end;

procedure scanner;
{*** zeigt die sichtbaren Satelliten ***}
var i, anz, len : byte;
    num : array[0..253] of byte;
    a   : array[0..253] of word;
    e   : array[0..253] of byte;
    az, el, s : string;
begin
  key:=chr(0);
  if sat>1 then begin
    anz:=0;
    tref:=0;
    len:=7;
    systime;
    for i:=1 to sat do begin
      berech(i);
      if elevation>=0 then begin
        inc(anz);
        num[anz]:=i;
        a[anz]:=round(azimut);
        e[anz]:=round(elevation);
        if length(satnam[i])>len then len:=length(satnam[i])
      end
    end;
    if anz>0 then begin
      len:=len+7;
      openwindow(midx-4*len-5,midy-10-5*anz,midx+4*len+5,midy+10+5*anz,14,0,15);
      if not winerror then begin
        setcolor(15);
        if anz>1 then outtextxy(0,0,'sichtbar sind:')
         else outtextxy(0,0,'sichtbar ist:');
        setcolor(14);
        for i:=1 to anz do begin
          str(a[i]:3,az);
          str(e[i]:2,el);
          s:=az+'/'+el+' '+satnam[num[i]];
          outtextxy(0,3+10*i,s)
        end;
        waitkey;
        closewindow
      end;
      key:='@';
    end else begin
      openwindow(midx-73,midy-15,midx+73,midy+15,14,0,15);
      if not winerror then begin
        outtextxy(5,1,'Derzeit ist kein');
        outtextxy(1,12,'Satellit sichtbar');
        waitkey;
        closewindow
      end;
      key:='@'
    end
  end
end;

procedure datamenu;
{*** Auswahl eines Satelliten, wenn mehrere vorhanden ***}
var i, j, mw, len, len2, basis : byte;

  function eingabe:byte;
  {*** Tastaturabfrage ***}
  var k : byte;
  begin
    repeat
      {$I-}
      while keypressed do key:=readkey;
      repeat until keypressed;
      key:=readkey;
      {$I+}
      k:=ord(upcase(key));
      if k in [65..75] then k:=k-55 else begin
        if k in [49..57] then k:=k-48 else begin
          k:=64;
          case key of
            #0  : begin
                    key:=readkey;
                    case key of
                      #72 : k:=254;
                      #73 : k:=254;
                      #80 : k:=255;
                      #81 : k:=255;
                    end
                  end;
            #27 : k:=mw
            else begin
              scanner;
              setrelativviewport(24,10,2*len-10,210)
            end
          end
        end
      end;
    until (k in [0..20]) or (k>253);
    key:=' ';
    eingabe:=k
  end;

begin
  key:=' ';
  mw:=wahl;
  if sat=1 then begin
    if wahl=1 then begin
      openwindow(midx-73,midy-15,midx+73,midy+15,14,0,15);
      if not winerror then begin
        outtextxy(1,1,'Derzeit ist keine');
        outtextxy(1,12,'Auswahl vorhanden');
        waitkey;
        closewindow
      end
    end else wahl:=1
  end else begin
    tref:=0;
    if sat<21 then begin
      len:=17;
      for i:=1 to sat do if length(satnam[i])>len then len:=length(satnam[i]);
      len:=4*len+17;
      i:=5*sat+10;
      openwindow(midx-len,midy-i,midx+len,midy+i,14,0,15);
      if not winerror then begin
        setcolor(15);
        outtextxy(1,1,'Welchen Satelliten ?');
        setcolor(14);
        systime;
        for i:=1 to sat do begin
          berech(i);
          if not (elevation<0) then setcolor(15);
          if i<10 then outtextxy(1,10*i+3,chr(i+48))
           else outtextxy(1,10*i+3,chr(i+55));
          outtextxy(9,10*i+3,': '+satnam[i]);
          setcolor(14)
        end;
        repeat
          wahl:=eingabe;
        until wahl<succ(sat);
        closewindow
      end else wahl:=mw
    end else begin
      basis:=0;
      len:=17;
      for i:=1 to sat do if length(satnam[i])>len then len:=length(satnam[i]);
      len:=4*len+17;
      openwindow(midx-len,midy-110,midx+len,midy+110,14,0,15);
      if not winerror then begin
        setcolor(15);
        outtextxy(1,1,'Welchen Satelliten ?');
        setcolor(14);
        for i:=1 to 20 do begin
          if i<10 then outtextxy(1,10*i+3,chr(i+48)+':')
          else outtextxy(1,10*i+3,chr(i+55)+':')
        end;
        setrelativviewport(24,10,2*len-10,210);
        repeat
          systime;
          for i:=1 to 20 do begin
            berech(i+basis);
            if not (elevation<0) then setcolor(15);
            outtextxy(1,10*i-7,satnam[i+basis]);
            setcolor(14)
          end;
          if basis>0 then outtextxy(winmaxx-8,10,#24);
          if basis<sat-20 then outtextxy(winmaxx-8,193,#25);
          wahl:=eingabe;
          if wahl>253 then begin
            if (wahl=254) and (basis>0) then begin
              if basis<20 then basis:=0 else basis:=basis-20
            end;
            if (wahl=255) and (basis<sat-20) then begin
              if basis>sat-40 then basis:=sat-20 else basis:=basis+20
            end;
            clearviewport
          end else wahl:=wahl+basis;
        until wahl<succ(sat);
        closewindow;
      end else wahl:=mw
    end
  end
end;

procedure illumination(dtime:extended);
{******************************************
Eingaben: aA, dA (Lage des Satelliten relativ zur Bahnebene)
          alphaSo, deltaSo (Lage der Bahnebene zur Sonne)
Ausgaben: ill (Illumination)
Grenzen: 1.1.1987 - 31.12.2000
geschrieben von DL1TV, veroeffentlicht im AMSAT-Journal 2/90
*******************************************}
var dSbm, dAbm, saa : extended;
begin
  solephemeriden(dtime);
  dSbm:=deltaSo*bm;
  dAbm:=satdA[wahl]*bm;
  saa:=sin(dSbm)*sin(dAbm)+cos(dSbm)*cos(dAbm)*cos((alphaSo-sataA[wahl])*bm);
  ill:=cos(arcsin(saa))
end;

function eklipse:boolean;
{*******************************************
Eingaben: alphaSo, deltaSo (Stand zur Sonne)
          sspb, sspl, sspe (Lage des Satelliten)
Ausgabe:  eklipse (true) und ill=0 (keine Illumination)
Grenzen: 1.1.1987 - 31.12.2000
Berechnet, ob sich der Satellit, der zuletzt mit berech(a)
berechnet wurde, im Erdschatten befindet.
Vorraussetzung dafuer ist die Kenntnis der Lage im Raum (aLON/aLAT).
Geschrieben von DL1TV, veroeffentlicht im AMSAT-Journal.
*******************************************}
var lamSubSoW, sspbbm, dSbm, cA, A, Ac : extended;

begin
  lamSubSoW:=GrtWi+360-alphaSo;
  sspbbm:=sspb*bm;
  dSbm:=deltaSo*bm;
  cA:=sin(dSbm)*sin(sspbbm)+cos(dSbm)*cos(sspbbm)*cos((lamSubSoW+sspl)*bm);
  A:=arccos(cA)/bm;           { Schattenwinkel zur Best. d. Eklipse }
  Ac:=90+arccos(1/sspe)/bm;   { Kritischer Bestrahl.wkl. }
  if A>Ac then eklipse:=true else eklipse:=false
end;

procedure info;
{*** gibt einige weitere Bahninfos aus ***}
var   ma, mma, t, th, tmin, tum : extended;
      i, j{, h, min, sec, s100}   : word;
      s, b                      : string;

  function xpos(a:string):byte;
  begin
    xpos:=4+i-(succ(8*length(a)) div 2)
  end;

  function perigeum:string;
  var sm, sh : string[2];
  begin
    th:=tuml(tref)*ma;
    tref:=t+th;
    th:=stunde+24*th;
    berech(wahl);
    tmin:=minute+frac(th)*60;
    if tmin>=60 then begin
      th:=th+trunc(tmin/60);
      tmin:=60*frac(tmin/60)
    end;
    str(trunc(tmin):2,sm);
    if tmin<10 then sm[1]:='0';
    th:=24*frac(th/24);
    if th<10 then str(int(th):1:0,sh) else str(int(th):2:0,sh);
    s:='Perigum um '+sh+':'+sm+' Uhr';
    perigeum:=s
  end;

  function apogeum:string;
  var sm, sh : string[2];
  begin
    th:=tuml(tref)*ma;
    tref:=t+th;
    th:=stunde+24*th;
    berech(wahl);
    tmin:=minute+frac(th)*60;
    if tmin>=60 then begin
      th:=th+trunc(tmin/60);
      tmin:=60*frac(tmin/60)
    end;
    str(trunc(tmin):2,sm);
    if tmin<10 then sm[1]:='0';
    th:=24*frac(th/24);
    if th<10 then str(int(th):1:0,sh) else str(int(th):2:0,sh);
    s:='Apogum um '+sh+':'+sm+' Uhr';
    apogeum:=s
  end;

  function aufgang:string;
  var tu, tob, eu, eo : extended;
      a : string;
      i : word;
  begin
    if pos('-',s)=0 then begin
      tob:=t;
      tref:=tob;
      berech(wahl);
      eo:=elevation;
      t:=tuml(tref)/256;
      i:=0;
      repeat
        tu:=tob;
        eu:=eo;
        tob:=tu+t;
        tref:=tob;
        berech(wahl);
        eo:=elevation;
        if eo<-10 then t:=tuml(tref)/16 else if eo<eu then t:=tuml(tref)/64 else t:=tuml(tref)/256;
        inc(i);
      until not (eo<0) or (i=1000) or keypressed;
      if (i=1000) or keypressed then a:='--:--' else begin
        a:='';
        while tob-tu>0.0001 do begin
          t:=tu+(tob-tu)/2;
          tref:=t;
          berech(wahl);
          if elevation<0 then begin
            tu:=t;
            eu:=elevation
          end else begin
            tob:=t;
            eo:=elevation
          end
        end;
        t:=24*frac(tu+(tob-tu)/2-zeitzone/24);
        if t<10 then str(int(t):1:0,b) else str(int(t):2:0,b);
        a:=b+':';
        t:=trunc(60*frac(t));
        str(t:2:0,b);
        if t<10 then b[1]:='0';
        a:=a+b
      end;
      t:=tob
    end else a:='--:--';
    aufgang:='Aufgang um '+a+' Uhr'
  end;

  function untergang:string;
  var tu, tob, eu, eo : extended;
      a : string;
      i : word;
  begin
    if pos('-',s)=0 then begin
      tob:=t;
      tref:=tob;
      berech(wahl);
      eo:=elevation;
      t:=tuml(tref)/256;
      i:=0;
      repeat
        tu:=tob;
        eu:=eo;
        tob:=tu+t;
        tref:=tob;
        berech(wahl);
        eo:=elevation;
        if eo>10 then t:=tuml(tref)/16 else if eu<eo then t:=tuml(tref)/64 else t:=tuml(tref)/256;
        inc(i);
      until (eo<0) or (i=1000) or keypressed;
      if (i=1000) or keypressed then a:='--:--' else begin
        a:='';
        while tob-tu>0.0001 do begin
          t:=tu+(tob-tu)/2;
          tref:=t;
          berech(wahl);
          if elevation<0 then begin
            tob:=t;
            eo:=elevation
          end else begin
            tu:=t;
            eu:=elevation
          end
        end;
        t:=24*frac(tu+(tob-tu)/2-zeitzone/24);
        if t<10 then str(int(t):1:0,b) else str(int(t):2:0,b);
        a:=b+':';
        t:=trunc(60*frac(t));
        str(t:2:0,b);
        if t<10 then b[1]:='0';
        a:=a+b
      end;
      t:=tob
    end else a:='--:--';
    untergang:='Untergang um '+a+' Uhr'
  end;

  function richtung:string;
  var a : integer;
  begin
    if pos('-',s)=0 then begin
      a:=round(azimut);
      if a<10 then str(a:1,s)
       else if a<100 then str(a:2,s) else str(a:3,s)
    end else s:='---';
    richtung:='bei '+s+' Grad'
  end;

  function hoehe:string;
  begin
    berech(wahl);
    str(sspr,s);
    s:='in '+s+' km Hhe';
    hoehe:=s
  end;

begin
  if wahl=255 then begin
    s:='Der Mond';
    j:=25
  end else begin
    s:='Satellit '+satnam[wahl];
    j:=73
  end;
  i:=succ(8*length(s)) div 2;
  if i<100 then i:=100;
  openwindow(midx-i-8,midy-j,midx+i+8,midy+j,14,0,15);
  if not winerror then begin
    j:=xpos(s);
    setcolor(15);
    outtextxy(j,3,s);
    setcolor(14);
    systime;
    tref:=time;
    if wahl=255 then tum:=27.32166 else tum:=tuml(tref);
    s:='Umlaufzeit ';
    if tum>=2 then begin
      str(tum:5:4,b);
      while b[1]=' ' do delete(b,1,1);
      s:=s+b+' Tage'
    end else begin
      if tum>0.5 then begin
        t:=int(24*tum);
        if t<100 then str(t:2:0,b) else str(t:3:0,b);
        s:=s+b;
        t:=round(60*frac(24*tum));
        str(t:2:0,b);
        if t<10 then b[1]:='0';
        s:=s+':'+b+' Stunden'
      end else begin
        t:=tum*1440;
        if t<100 then str(t:2:2,b) else str(t:3:2,b);
        s:=s+b+' Minuten'
      end
    end;
    j:=xpos(s);
    outtextxy(j,19,s);
    t:=tum*zf;
    while t>=360 do t:=t-360;
    if t<10 then str(t:1:2,s) else if t<100 then str(t:2:2,s) else str(t:3:2,s);
    s:='Bahnversatz '+s+' Grad';
    j:=xpos(s);
    outtextxy(j,31,s);
    if wahl<255 then begin
      tref:=0;
      berech(wahl);
      mma:=anomalie/256;
      tref:=time;
      t:=tref;
      if mma>0.5 then begin
        ma:=1-mma;
        s:=perigeum;
        j:=xpos(s);
        outtextxy(j,43,s);
        s:=hoehe;
        j:=xpos(s);
        outtextxy(j,55,s);
        j:=j+8*length(s)-22;
        ma:=ma+0.5;
        s:=apogeum;
        j:=xpos(s);
        outtextxy(j,67,s);
        s:=hoehe;
        j:=xpos(s);
        outtextxy(j,79,s);
        j:=j+8*length(s)-22
      end else begin
        ma:=0.5-mma;
        s:=apogeum;
        j:=xpos(s);
        outtextxy(j,43,s);
        s:=hoehe;
        j:=xpos(s);
        outtextxy(j,55,s);
        j:=j+8*length(s)-22;
        ma:=ma+0.5;
        s:=perigeum;
        j:=xpos(s);
        outtextxy(j,67,s);
        s:=hoehe;
        j:=xpos(s);
        outtextxy(j,79,s);
        j:=j+8*length(s)-22
      end;
      tref:=0;
      berech(wahl);
      if elevation<0 then begin
        s:=aufgang;
        j:=xpos(s);
        outtextxy(j,91,s);
        s:=richtung;
        j:=xpos(s);
        outtextxy(j,103,s);
        s:=untergang;
        j:=xpos(s);
        outtextxy(j,115,s);
        s:=richtung;
        j:=xpos(s);
        outtextxy(j,127,s)
      end else begin
        s:=untergang;
        j:=xpos(s);
        outtextxy(j,91,s);
        s:=richtung;
        j:=xpos(s);
        outtextxy(j,103,s);
        s:=aufgang;
        j:=xpos(s);
        outtextxy(j,115,s);
        s:=richtung;
        j:=xpos(s);
        outtextxy(j,127,s)
      end
    end;
    tref:=0;
    repeat
      if mentfernung>0 then begin
        systime;
        if wahl=255 then mond else berech(wahl);
        monitor
      end;
    until keypressed;
    closewindow;
    while keypressed do key:=readkey
  end;
  key:='@'
end;

procedure bahnberechnung;
{***************************
Hier gehts zur Sache.
Dies ist die Prozedur zur Bahnverfolgung in Echtzeit.
***************************}
var k : byte;
begin
  if test8087>1 then k:=1 else k:=0;
  sspi:=0;               {diese Variablen dienen der Feststellung}
  sspj:=0;               {irgendwelcher Aenderungen, die z.B. ausgegeben}
  msspl:=181;            {werden muessen, sowie der Erkennung der Erst-}
  msspb:=91;             {durchlaeufe zur Initialisierung diverser auch}
  msspr:=0;              {grafischer Routinen}
  mtag:=0;
  mminute:=61;
  morbit:=0;
  mazimut:=361;
  melevation:=91;
  manomalie:=0;
  mentfernung:=0;
  msquint:=181;
  squint:=181;
  mekl:=false;
  ekl:=false;
  sir:=0;
  key:='@';
  tref:=0;
  repeat
    systime;
    if wahl<254 then begin
      berech(wahl);
      illumination(0);
      if eklipse then begin
        ill:=0;
        ekl:=true
      end
    end else if wahl=255 then mond;
    ssph:=round((sspl+180)*x/360);
    sspv:=round(y-y/180*(sspb+90)+offset);
    if (ssph<>sspi) or (sspv<>sspj) then begin
      if sir>0 then sichtaus;
      sichtein;
      if k>0 then k:=1;
      setcolor(15);
      line(ssph-3,sspv-3,ssph+3,sspv+3);
      line(ssph-3,sspv+3,ssph+3,sspv-3);
      sspi:=ssph;
      sspj:=sspv
    end else begin
      if k=20 then begin
        k:=1;
        if sir>0 then sichtaus;
        sichtein
      end else if k>0 then inc(k);
      if getpixel(sspi,sspj)=15 then setcolor(0) else setcolor(15);
      line(sspi-3,sspj-3,sspi+3,sspj+3);
      line(sspi-3,sspj+3,sspi+3,sspj-3)
    end;
    if mentfernung=0 then begin
      grwritexy(8,1,'Uhr',14);
      if wahl<254 then grwritexy(36,1,satnam[wahl],14) else begin
        if wahl=255 then grwritexy(36,1,'Mond',14) else
        grwritexy(36,1,'Sonne',14)
      end;
      grwritexy(1,uz,'                        MA:   0/        Antenne:    /     Entfernung: ',14)
    end;
    monitor;
    if keypressed then key:=readkey;
    if key='?' then begin
      sichtaus;
      fastblock(banf,bend);
      info;
      key:='@';
      sichtein
    end;
  until key<>'@';
  dellines;
  sichtaus;
  fastblock(banf,bend);
  mentfernung:=0;
  tref:=0
end;

procedure bahnhoehe;
{*** gibt die Bahnhoehe des naechsten Umlaufs grafisch aus ***}
var   a, p                : string;
      ta, ma, tum, dt, dp : extended;
      i                   : word;
begin
  while keypressed do key:=readkey;
  if wahl<254 then begin
    openwindow(midx-200,midy-130,midx+200,midy+130,14,0,15);    {400*260}
    if not winerror then begin
      setcolor(15);
      outtextxy(1,1,'Bahnhhe '+satnam[wahl]);
      setcolor(14);
      line(10,16,10,220);
      line(6,220,390,220);
      systime;
      tref:=time;
      berech(wahl);
      ma:=anomalie/256;
      if ma<0.5 then ma:=0.5-ma else ma:=1.5-ma;
      tum:=tuml(tref);
      tref:=tref+ma*tum;
      berech(wahl);
      ta:=sspr;
      str(sspr,a);
      a:='Apogum '+a+' km';
      outtextxy(10,227,a);
      dp:=200/sspr;
      tref:=time;
      tref:=trefp;
      berech(wahl);
      str(sspr,p);
      p:='Perigum '+p+' km';
      outtextxy(10,238,p);
      dt:=tuml(time);
      if dt>0.5 then begin
        str(dt:5:10,p);
        while p[1]=#32 do delete(p,1,1);
        p:='Umlaufzeit '+copy(p,1,7)+' Tage'
      end else begin
        str(1440*dt:5:10,p);
        while p[1]=#32 do delete(p,1,1);
        p:='Umlaufzeit '+copy(p,1,8)+' min'
      end;
      outtextxy(200,227,p);
      str(satecc[wahl]:1:7,p);
      p:='Exzentrizitt '+p;
      outtextxy(200,238,p);
      if ta<1000 then begin
        if ta>140 then begin
          i:=220-round(150*dp);
          setcolor(4);
          line(11,i,390,i);
          if ta>160 then i:=i-8 else i:=i+2;
          outtextxy(12,i,'150 km')
        end;
        if ta>60 then begin
          i:=220-round(50*dp);
          setfillstyle(1,1);
          bar(11,i,390,219)
        end;
        i:=220-round(10*dp);
        setfillstyle(1,9);
        bar(11,i,390,219);
        setcolor(14)
      end else if ta<10000 then begin
        i:=220-round(150*dp);
        setcolor(1);
        line(11,i,390,i);
        if i<25 then i:=i+2 else i:=i-8;
        outtextxy(11,i,'150 km');
        setcolor(14)
      end;
      dt:=tum/360;
      tref:=time;
      i:=11;
      repeat
        berech(wahl);
        putpixel(i,220-round(sspr*dp),15);
        tref:=tref+dt;
        inc(i);
      until keypressed or (i>390);
      tref:=0;
      waitkey;
      closewindow
    end
  end;
  key:='@'
end;

procedure illprognose;
{***************************************
Gibt die Illumination der naechsten 360 Umlaeufe grafisch aus.
Vorraussetzung ist die Kenntnis der Lage des Satelliten im Raum (aLON/aLAT).
***************************************}
var t, dt : extended;
    i : word;
    s : string;
begin
  while keypressed do key:=readkey;
  if (wahl<254) then begin
    if (sataa[wahl]>0) or (satda[wahl]>0) then begin
      openwindow(midx-200,midy-130,midx+200,midy+130,14,0,15);    {400*260}
      if not winerror then begin
        setcolor(15);
        outtextxy(1,1,'Illumination '+satnam[wahl]);
        setcolor(14);
        line(10,16,10,234);
        line(6,230,388,230);
        line(381,226,381,234);
        outtextxy(140,240,'1 Pixel/Orbit');
        str(orbit,s);
        s:='Orbit '+s;
        outtextxy(3,240,s);
        str(orbit+370,s);
        s:='Orbit '+s;
        outtextxy(389-8*length(s),240,s);
        setcolor(4);
        line(11,30,388,30);
        line(11,130,388,130);
        outtextxy(12,21,'100%');
        outtextxy(12,121,'50%');
        outtextxy(12,221,'0%');
        setcolor(14);
        systime;
        berech(wahl);
        t:=0;
        dt:=tuml(time);          { <--- 1 Pixel/Umlauf }
        i:=11;
        repeat
          illumination(t);
          putpixel(i,230-round(200*ill),15);
          inc(i);
          t:=t+dt;
        until keypressed or (i>388);
        waitkey;
        closewindow
      end
    end else begin
      openwindow(midx-97,midy-15,midx+97,midy+15,14,0,15);
      if not winerror then begin
        outtextxy(1,1,'Die Lage des Satelliten');
        outtextxy(25,12,'ist nicht bekannt');
        waitkey;
        closewindow
      end
    end
  end;
  key:='@'
end;

procedure eklprognose;
{*** gibt die Daten der naechsten Eklipsen als Tabelle aus ***}
var t, dt : extended;
    i, j : word;
    p : byte;
    e : boolean;
    r : string[3];
    s : string[23];
    u, v : string[2];

  function zeit(t:extended):string;
  {*** gibt den Nachkommawert von t als Uhrzeit-String zurueck ***}
  var u, v : string[2];
      j : real;
      h, m, k : integer;
  begin
    j:=24*frac(t-zeitzone/24);
    h:=trunc(j);
    m:=trunc(60*frac(j));
    str(m,v);
    if v='60' then begin
      v:='00';
      h:=succ(h) mod 24
    end;
    if length(v)=1 then v:='0'+v;
    str(h,u);
    if u='24' then u:='0';
    if length(u)=1 then u:=' '+u;
    zeit:=u+':'+v
  end;

  procedure inctag;
  {****************
  Erhoeht den Tag, der in der Prozedur aus Geschwindigkeitsgruenden
  mitgezaehlt wird, um eins und korrigiert das Datum entsprechend.
  ****************}
  begin
    tag:=tag+1;
    if tag>28 then begin
      if monat=2 then begin
        if (jahr=2000) or (jahr and 3>0) then tag:=30;
        if tag>29 then begin
          monat:=3;
          tag:=1
        end
      end;
      if tag>30 then begin
        if (tag=32) or (monat=4) or (monat=6) or (monat=9) or (monat=11) then begin
          monat:=monat+1;
          tag:=1;
          if monat=13 then begin
            monat:=1;
            jahr:=jahr+1
          end
        end
      end
    end
  end;

begin
  while keypressed do key:=readkey;
  if (wahl<254) then begin
    openwindow(midx-153,midy-127,midx+153,midy+127,14,0,15);    {306*254}
    if not winerror then begin
      setcolor(15);
      outtextxy(1,1,'Eklipsen '+satnam[wahl]);
      setcolor(14);
      outtextxy(9,13,'Orbit Datum   Zeit MA       Zeit MA');
      systime;
      tref:=time;
      berech(wahl);
      t:=0;
      dt:=tuml(tref)/256;          { <--- Auflsung 1 MA }
      i:=27;
      e:=false;
      repeat
        solephemeriden(t);
        if eklipse and not e then begin
          str(orbit,s);
          while length(s)<6 do s:=' '+s;
          str(trunc(anomalie),r);
          while length(r)<3 do r:='0'+r;
          str(tag,u);
          while length(u)<2 do u:=' '+u;
          str(monat,v);
          while length(v)<2 do v:='0'+v;
          s:=s+' '+u+'.'+v+'. '+zeit(tref)+' '+r;
          outtextxy(1,i,s);
          p:=length(s);
          e:=true
        end;
        if e and not eklipse then begin
          anomalie:=anomalie-1;
          if anomalie<0 then begin
            anomalie:=255;
            orbit:=orbit-1
          end;
          str(trunc(anomalie),r);
          while length(r)<3 do r:='0'+r;
          s:=' bis '+zeit(tref)+' '+r;
          outtextxy(succ(8*p),i,s);
          i:=i+10;
          e:=false
        end;
        str(orbit,s);
        str(round(int(anomalie)),r);
        s:='Orbit '+s+' MA '+r+'  ';
        grwritexy(1,1,s,15);
        t:=t+dt;
        tref:=tref+dt;
        if frac(tref-zeitzone/24)<dt then inctag;
        berech(wahl);
      until keypressed or (i>240);
      tref:=0;
      dellines;
      waitkey;
      closewindow;
      systime
    end
  end;
  key:='@'
end;

procedure mainwin;
{*** HAUPTMENUE ***}
var ok, esc : boolean;

  procedure win1;
  var farbe : byte;
  begin
    openwindow(midx-85,midy-60,midx+85,midy+60,14,0,15);
    if not winerror then begin
      outtextxy(1,1,'SAT-Master Hauptmen');
      outtextxy(1,13,'1: Bahnberechnung');
      outtextxy(1,23,'2: Auswahlmen');
      outtextxy(1,33,'3: Spurkarte');
      outtextxy(1,43,'4: Scanner/suchen');
      outtextxy(1,53,'5: Weitere Bahninfos');
      outtextxy(1,63,'6: Mondberechnung');
      outtextxy(1,73,'7: Bahnkurve/Hhe');
      outtextxy(1,83,'8: Illumination');
      outtextxy(1,93,'9: Eklipsen');
      outtextxy(1,103,'0: Ende')
    end else begin   {schwere Probleme wollen erwartet werden,}
      closegraph;    {damit sie ausbleiben...}
      delay(1);
      writeln('Heap overflow (Speicher voll)');
      writeln('Programm abgebrochen...');
      halt(0)
    end
  end;

begin
  win1;
  ok:=false;
  esc:=false;
  repeat
    repeat until keypressed;
    if key=chr(0) then begin
      while keypressed do key:=readkey;
      key:='@'
    end;
    key:=upcase(readkey);
    if key<>#27 then esc:=false;
    case key of
      '0' : begin
              wahl:=0;
              ok:=true
            end;
      '1' : ok:=true;
      '2' : begin
              datamenu;
              ok:=true
            end;
      '3' : begin
              if sat<254 then begin
                closewindow;
                trackmap;
                win1
              end
            end;
      '4' : scanner;
      '5' : info;
      '6' : begin
              wahl:=255;
              ok:=true
            end;
      '7' : bahnhoehe;
      '8' : illprognose;
      '9' : eklprognose;
      'R' : begin
              rwin;
              win1
            end;
      'S' : begin
              swin;
              win1
            end;
      'V' : begin
              vwin;
              win1
            end;
      #27 : begin
              if esc then begin
                wahl:=0;
                ok:=true
              end else esc:=true
            end;
      else fehlt
    end;
  until ok;
  key:='@';
  closewindow
end;

begin
  clrscr;                            { aufraeumen fuer Fehlermeldungen }
  systime;                           { Systemzeit holen }
  if jahr=1980 then begin            { Uhr gestellt? }
    writeln('Bitte stelllen Sie erst Ihre Systemuhr...');
    writeln;
    writeln('SAT-Master abgebrochen...');
    halt(1)
  end;
  writeln('SAT-Master wird geladen - bitte warten...'); { er lebt noch... }
  writeln;
  pi2:=2*pi;                         { muss man ja nicht dauernd berechnen }
  getmem(memssp,55);                 { Speicher fuer SSP }
  sat:=0;                            { kein Satellit }
  sspi:=0;
  sspj:=0;
  tref:=0;
  textbackground(0);                 { Hintergrund schwarz }
  loaddata;                          { Daten laden }
  if amsat then begin                { Rotorinterface einrichten }
    druckport:=mem[$40:lpt*2+8]+256*mem[$40:lpt*2+9]; {was steht im BIOS-RAM?}
    druckport:=druckport and $3FF;   { auf 3FFh (legale I/O) maskieren }
    if (druckport<$200) then begin   { Gefahr! }
      if druckport=0 then writeln('LPTadresse nicht definiert.')
      else writeln('LPTadresse unter 200h gefunden.');
      writeln;
      writeln('SAT-Master abgebrochen...');
      halt(1)
    end else port[druckport]:=0
  end;
  if sat=0 then loaddummy;     { keine Daten gefunden }
  setgraph;                    { Grafikmodus aufrufen }
  x:=getmaxx;                  { Wichtig fuers weitere Programm, }
  midx:=x div 2;               { damit Texte und Fenster mittig }
  y:=getmaxy;                  { ausgegeben werden koennen }
  midy:=y div 2;               { und die Aufloesung }
  y:=y-2*offset;               { des Modus bekannt ist. }
  map;                         { Weltkarte ausgeben }
  grwritexy(27,1,'Willkommen bei SAT-Master v',15);
  grwritexy(54,1,version,15);
  wahl:=0;                     { keine Auswahl }
  datamenu;                    { also fragen }
  if wahl>0 then begin         { sinnvoll gewaehlt? }
    dellines;                  { Willkommen-Text loeschen }
    while wahl>0 do begin      { Programm, solange sinnvolle Wahl }
      bahnberechnung;          { erste Ausgabe, bis Taste }
      if key<>'@' then mainwin  { dann Hauptmenue... }
    end
  end;
  closegraph;                  { aufraeumen }
  clrscr;
  writeln;
  writeln('SAT-Master v',version)
end.
