{$A+,L+,R+,S+,I+,F-,O-,N+,E+,L+,V+,B+,M 16384 0 655360}
program smallsat;

uses dos;

const  copyright : string[15] = '(C) DB1HZ 1991';
       version   : string[4]  = '1.3';
       maxsat    : byte       = 9;
       bm        : extended   = 0.0174532925199327; {pi/180}
       fp        : extended   = 100.2907;           {Fruehlingspunkt 1978}
       zf        : extended   = 360.985647332099;   {Erdrotation in Grad pro Tag gegen Himmel}
       er        : extended   = 6378.145;           {Erdradius in km}
       pi2       : extended   = 6.28318531;         {2*pi}

var    sat, wahl                                     : byte;
       sek100, sekunde, minute, stunde, tag,
       wtag, monat, jahr, dummy                      : word;
       j, zeitzone, amsattag                         : integer;
       entfernung, sspr, orbit                       : longint;
       azimut, elevation, anomalie, squint, GrtWi, ill,
       sspl, sspb, sspe, laenge, breite, alphaSo, rechnerzeit,
       deltaSo                                       : extended;
       s                                             : string;
       v                                             : string[2];
       satet, satincl, satraan, satecc, satargp,
       satma, satmm, satemm, sataa, satda   : array[1..9] of extended;
       satuml                               : array[1..9] of longint;
       satnam                               : array[1..9] of string[30];


function hoch(wert,expon:extended):extended;
begin
  hoch:=exp(ln(wert)*expon)
end;

function arcsin(wert:extended):extended;
begin
  arcsin:=arctan(wert/(sqrt(1-wert*wert)+1e-18))
end;

function arccos(wert:extended):extended;
begin
  arccos:=pi/2-arctan(wert/(sqrt(1-wert*wert)+1e-18))
end;

procedure systime;
begin
  getdate(jahr,monat,tag,wtag);
  gettime(stunde,minute,sekunde,sek100);
  rechnerzeit:=stunde/24+minute/1440+sekunde/86400+sek100/8640000
end;

function doy:extended;
{*********************
Eingabe: keine (hoechstens Systemzeit)
Ausgabe: Day Of Year (Anzahl der Tage seit 1.Januar mit genauer Uhrzeit als
         Nachkomma-Wert)
Umsetzung des Monats mittels Tabelle aus Geschwindigkeitsgruenden.
 *********************}
const mt : array[1..12] of word = (0,31,59,90,120,151,181,212,243,273,304,334);
var   zt : extended;
      jw : word;
begin
  zt:=mt[monat]+tag+rechnerzeit+zeitzone/24;
  if (monat>2) and (0=jahr and 3) and (jahr<>2000) then zt:=zt+1;
  doy:=zt
end;

function time:extended;
var i  : byte;
    jw : word;
begin
  jw:=jahr-1900;
  if jw>1999 then i:=1 else i:=0;
  time:=365*jw+((jw+3) DIV 4)+doy-i
end;

function amsattime(a:byte):extended;
{**********************
Eingabe: a (Nummer des Satelliten)
Ausgabe: Anzahl der Tage vom 1.1.1900 bis zum Zeitpunkt der Bahndaten
 **********************}
var ej : word;
    et, zg : extended;
begin
  ej:=round(satet[a]) div 1000;           {Epoche Jahr}
  if ej<80 then ej:=ej+100;               {Datum im 21. Jahrhundert!}
  et:=1000*frac(satet[a]/1000);           {Epoche Tag}
  zg:=et+365*ej+((ej+3) DIV 4)-satma[a]/(360*satmm[a]);  {Zeit Referenzperigum absolut}
  if ej>99 then zg:=zg-1;                 {2000 ist KEIN Schaltjahr!}
  amsattime:=zg
end;

procedure berech(a:byte);
var   zdx,zdy,zdz,zmm,za,zb,zc,zd,zh,zi,zsi,zci,zj,zk,zl,zm,zn,
      zo,zp,zq,zr,zs,zt,zu,zv,zw,zx,zy,zz,dv,tv,av : extended;

  procedure schleife(var zp,zr,zx,zy:extended);
  begin
    zr:=sqrt(zx*zx+zy*zy);
    if zx=0 then if zy<0 then zp:=-90 else zp:=90
    else begin
      zp:=arctan(zy/zx)/bm;
      if zx<=0 then if zy>0 then zp:=zp+180 else zp:=zp-180
    end
  end;

begin
  zdz:=breite*bm;
  zt:=time;
  zd:=zt-amsattime(a);                {Differenzzeit seit Referenzperigaeum}
  amsattag:=trunc(zt-28451);
  zmm:=satmm[a]+satemm[a]*zd;         {akt. Mean Motion}
  zn:=pi2*zmm;                        {Winkelgeschw.}
  za:=42220*hoch(zmm,-2/3)/er;        {gr. Bahnhalbachse nach Kepler: U^2=T^3}
  zz:=hoch(za,3.5)*sqr(1-satecc[a]*satecc[a]);
  zi:=satincl[a]*bm;                  {Inkl. im Bogenmass}
  zsi:=sin(zi);                       {sin(inkl)}
  zci:=cos(zi);                       {cos(inkl)}
  zq:=-9.9999999999*zci/zz;           {dArgPer pro Tag}
  zv:=(25*zci*zci-5)/zz;              {dRAAN pro Tag}
  zo:=satraan[a]+zd*zq;               {aktueller RAAN}
  zw:=satargp[a]+zd*zv;               {aktuelles ArgPer}
  zk:=zo-fp-zt*zf-laenge;             {Laengendifferenz}
  zm:=zd*zn;                          {Differenzumlaeufe}
  zr:=satecc[a];                      {Naeherung nach Kepler}
  zp:=zm;                             {im Bogenmass}
  repeat                         {Algorithmus nach Dr.Karl Meinzer/AMSAT-DL}
    zh:=(zm-zp+zr*sin(zp))/(1-zr*cos(zp));  {siehe CQ-DL 10/1985 S.577ff}
    zp:=zp+zh;
  until abs(zh)<1E-10;
  zy:=sin(zp)*sqrt(1-satecc[a]*satecc[a]);
  zx:=cos(zp)-satecc[a];
  schleife(zp,zr,zx,zy);
  zr:=za*zr;
  zp:=bm*(zp+zw);
  zy:=zr*sin(zp);
  zh:=zr*cos(zp);
  zp:=zi;
  zs:=zy*zsi;
  zx:=zy*zci;
  schleife(zp,zr,zh,zx);
  zp:=zp+zk;
  zk:=zp+laenge;
  zu:=zr;
  zp:=zp*bm;
  zh:=zr*sin(zp);
  zz:=zs-sin(zdz);
  zx:=zr*cos(zp)-cos(zdz);
  schleife(zp,zr,zx,zz);
  zp:=bm*(zp-breite+90);
  zj:=zr*sin(zp);
  zx:=zr*cos(zp);
  schleife(zp,zr,zx,zh);
  zc:=180-zp;
  azimut:=round(10*zc)/10;
  zx:=zr;
  schleife(zp,zr,zx,zj);            {soweit Fertig}
  elevation:=round(10*zp)/10;
  zm:=zm/pi2;                       {akt. Bahnpunkt}
  anomalie:=round(2560*frac(zm))/10;
  orbit:=trunc(zm)+satuml[a];
  zdx:=zp*bm;
  if (sataa[a]>0) or (satda[a]>0) then begin    {Squintwinkel}
    dv:=arcsin(sin(zdx)*sin(zdz)+cos(zdx)*cos(zdz)*cos(zc*bm));
    tv:=arccos((sin(zdx)-sin(zdz)*sin(dv))/(cos(zdz)*cos(dv)))/bm;
    if zc<180 then tv:=360-tv;
    av:=360+fp+(zf-360)*int(zt)+zf*frac(zt)+laenge-tv;
    squint:=round(arccos(sin(satda[a]*bm)*sin(dv)
            +cos(satda[a]*bm)*cos(dv)*cos((sataa[a]-av)*bm))/bm)
  end else squint:=361;             {siehe AMSAT-DL 2/90 S.10ff - DL1TV}
  zk:=360*frac(zk/360);             {SubSatellitenPunkt berechnen}
  schleife(zp,zr,zu,zs);
  if zk<-180 then zk:=zk+360;       {Anpassen fuer Ausgabe}
  if zk>180 then zk:=zk-360;
  sspl:=round(100*zk)/100;
  sspb:=round(100*zp)/100;
  sspr:=round(er*(zr-1));
  sspe:=zr;
  zdy:=cos(zdx);                    {Entfernung zum Satelliten}
  entfernung:=round(er*(sqrt(zr*zr-zdy*zdy)-sin(zdx)));
                      {Dreieck Sat-QTH-Erde mit Winkel Elevation...}
end;

procedure loaddummy;
begin                          { Dummy Bahndaten AO13 }
  sat          := 1;
  satnam[sat]  := 'Amsat Oscar 13';
  satet[sat]   := 91053.11379;        {22.2.1991}
  satincl[sat] := 56.83;
  satraan[sat] := 109.09;
  satecc[sat]  := 0.7128;
  satargp[sat] := 247.86;
  satma[sat]   := 26.43;
  satmm[sat]   := 2.09704;
  satemm[sat]  := -0.0000013;
  satuml[sat]  := 2061;
  sataa[sat]   := 157.869;
  satda[sat]   := 52.896
end;

procedure konvert(alon, alat:extended; a:byte);
var dabm, i, si, ci, w1, w2, da, aa : extended;
begin                               {Umwandlung von Alon/Alat nach aA/dA}
  i:=satincl[a]*bm;                 {Siehe AMSAT-DL 2/90 S.13}
  w2:=450-satargp[a]-alon;
  da:=arcsin(cos(i)*sin(alat*bm)+sin(i)*cos(alat*bm)*cos(w2*bm));
  w1:=arccos((sin(alat*bm)-cos(i)*sin(da))/(sin(i)*cos(da)))/bm;
  if sin(w2*bm)<0 then w1:=360-w1;
  sataA[a]:=w1+satraan[a]-90;
  satdA[a]:=da/bm
end;

procedure loaddata;
var  di : searchrec;
     i  : integer;
     f  : text;
     d, s, s1, s2  : string;
     j             : boolean;
     alon, alat, w : extended;

  function del(a:string):string;
  begin
    if pos(':',a)>0 then delete(a,1,succ(pos(':',a)));
    while ((pos(' ',a)=1) or (pos(' ',a)=length(a))) and (length(a)>0)
     do delete(a,pos(' ',a),1);
    del:=a
  end;

  function rd:extended;
  var  w : real;
       t : string;
  begin
    if not eof(f) and not j then begin
      readln(f,t);
      t:=del(t);
      val(t,w,i);
      if i<>0 then j:=true else rd:=w
    end
  end;

begin
  {$I-}
  assign(f,'SAT.DAT');
  reset(f);
  if ioresult=0 then begin
    if eof(f) then i:=1 else begin
      readln(f,d);
      d:=del(d);
      val(d,laenge,i);
      if (i=0) and not eof(f) then begin
        readln(f,d);
        d:=del(d);
        val(d,breite,i);
        if (i=0) and not eof(f) then begin
          readln(f,d);
          d:=del(d);
          val(d,zeitzone,i);
          if i<>0 then zeitzone:=0
        end else zeitzone:=0
      end else i:=1
    end;
  end else i:=1;
  if i>0 then begin
    laenge:=10;
    breite:=53.5;
    zeitzone:=-1
  end;
  close(f);
  {$I+}
  i:=ioresult;
  sat:=0;
  i:=0;
  findfirst('*.SAT',i,di);
  while (doserror=0) and (sat<maxsat) do begin
    d:=di.name;
    assign(f,d);
    {$I-}
    reset(f);
    if ioresult=0 then begin
      j:=false;
      inc(sat);
      if not eof(f) then readln(f,s) else j:=true;
      s:=del(s);
      satnam[sat]:=s;
      if satnam[sat]='' then satnam[sat]:=chr(64+sat)+'?';
      satet[sat]:=rd;
      if (satet[sat]<0) or (satet[sat]>=100000) then j:=true;
      satincl[sat]:=rd;
      if (satincl[sat]<0) or (satincl[sat]>180) then j:=true;
      satraan[sat]:=rd;
      if (satraan[sat]<0) or (satraan[sat]>360) then j:=true;
      satecc[sat]:=rd;
      if (satecc[sat]<0) or (satecc[sat]>=1) then j:=true;
      satargp[sat]:=rd;
      if (satargp[sat]<0) or (satargp[sat]>360) then j:=true;
      satma[sat]:=rd;
      if (satma[sat]<0) or (satma[sat]>360) then j:=true;
      satmm[sat]:=rd;
      if (satmm[sat]<=0) or (satmm[sat]>50) then j:=true;
      satemm[sat]:=rd;
      if satemm[sat]>1 then satemm[sat]:=0;
      if eof(f) then j:=true;
      satuml[sat]:=trunc(rd);
      if satuml[sat]<0 then j:=true;
      alon:=0;
      alat:=0;
      sataa[sat]:=0;
      satda[sat]:=0;
      while not eof(f) and not j do begin
        readln(f,s1);
        s2:=del(s1);
        val(s2,w,i);
        if i=0 then begin
          s1:=copy(s1,1,3);
          if s1='ALO' then alon:=w;
          if s1='ALA' then alat:=w;
          if s1='AA:' then sataa[sat]:=w;
          if s1='DA:' then satda[sat]:=w;
        end else j:=true
      end;
      if not j then begin
        if (alon<-180) or (alon>360) or (alat<-90) or (alat>180) then begin
          alon:=0;
          alat:=0
        end;
        if (sataa[sat]<-180) or (sataa[sat]>360) or (satda[sat]<-90) or
         (satda[sat]>180) then begin
          sataa[sat]:=0;
          satda[sat]:=0
        end;
        if (sataa[sat]=0) and (satda[sat]=0) and ((alon<>0) or (alat<>0))
         then konvert(alon, alat, sat)
      end else dec(sat)
    end;
    close(f);
    {$I+}
    findnext(di)
  end
end;

procedure solephemeriden(dtime:extended);    {von DL1TV}
  {Eingabe: dtime (Zeitoffset) }
  {Ausgaben: alphaSo, deltaSo (Sonnenstand), GrtWi }
  {Grenzen: 1.1.1987 - 31.12.2000}

const l0      : array[1987..2000] of real     = (279.12820, 278.89110,
                                                 279.63952, 279.40202,
                                                 279.16409, 278.92566,
                                                 279.67233, 279.43286,
                                                 279.19296, 278.95277,
                                                 279.69811, 279.45787,
                                                 279.21786, 278.97823);

      g0      : array[1987..2000] of real     = (356.41766, 356.16176,
                                                 356.89147, 356.63557,
                                                 356.37968, 356.12378,
                                                 356.85349, 356.59759,
                                                 356.34170, 356.08580,
                                                 356.81551, 356.55962,
                                                 356.30372, 356.04783);

      e       : array[1987..2000] of real     = (0.0167146, 0.0167142,
                                                 0.0167137, 0.0167133,
                                                 0.0167129, 0.0167125,
                                                 0.0167121, 0.0167116,
                                                 0.0167112, 0.0167108,
                                                 0.0167104, 0.0167100,
                                                 0.0167095, 0.0167091);

      epsilon : array[1987..2000] of real     = (23.44350, 23.44333,
                                                 23.44288, 23.44218,
                                                 23.44130, 23.44033,
                                                 23.43935, 23.43847,
                                                 23.43777, 23.43732,
                                                 23.43714, 23.43725,
                                                 23.43762, 23.43819);

      grtw0   : array[1987..2000] of real     = (99.12766, 98.89041,
                                                 99.63870, 99.40109,
                                                 99.16309, 98.92463,
                                                 99.67131, 99.43189,
                                                 99.19208, 98.95201,
                                                 99.69747, 99.45734,
                                                 99.21743, 98.97787);

      Kd  : extended = 0.9856473;
      Kdg : extended = 0.9856003;
      Kt  : extended = 15.04107;

var   j : word;
      s : string[5];
      Epoche, LSoStrich, LSo, g, taSo, sdSo,
      saa, dSbm, dAbm : extended;

begin
  Epoche:=Doy+dtime;
  j:=jahr;
  if (j>1986) and (j<2001) then begin
    g:=g0[j]+Kdg*Epoche;
    if g>360 then g:=g-360;
    LSoStrich:=L0[j]+Kd*Epoche;
    LSo:=LSoStrich+(2*E[j]*sin(g*bm)+1.25*sqr(E[j])*sin(2*g*bm))/bm;
    if LSo>360 then LSo:=LSo-360;
    if (LSo<>90) and (LSo<>270) then begin
      taSo:=sin(LSo*bm)*cos(Epsilon[j]*bm)/cos(LSo*bm);
      alphaSo:=arctan(taSo)/bm         { taSo:=tan(alphaSo*bm) }
    end else begin
      if LSo=90 then alphaSo:=90;
      if LSo=-90 then alphaSo:=-90
    end;
             { alpha im gleichen Quadranten wie LSo: }
    if cos(LSo*bm)<0 then alphaSo:=alphaSo+180        { II, III }
    else if alphaSo<0 then alphaSo:=alphaSo+360;        { IV }
    sdSo:=sin(LSo*bm)*sin(Epsilon[j]*bm);
    deltaSo:=arcsin(sdSo)/bm;          { sdSo:=sin(deltaSo*bm) }
    GrtWi:=GrtW0[j]+Kd*trunc(Epoche)+Kt*24*frac(Epoche);
    if GrtWi>360 then GrtWi:=GrtWi-360;
    if GrtWi>360 then GrtWi:=GrtWi-360
  end
end;

procedure illumination(dtime:extended);    {von DL1TV}
  {Eingaben: aA, dA (Lage des Satelliten) }
  {          alphaSo, deltaSo (Stand zur Sonne) }
  {Ausgaben: ill (Illumination) }
  {Grenzen: 1.1.1987 - 31.12.2000}

var dSbm, dAbm, saa : extended;
begin
  dSbm:=deltaSo*bm;
  dAbm:=satdA[wahl]*bm;
  saa:=sin(dSbm)*sin(dAbm)+cos(dSbm)*cos(dAbm)*cos((alphaSo-sataA[wahl])*bm);
  ill:=cos(arcsin(saa))
end;

function eklipse:boolean;   {von DL1TV}
  {Eingaben: alphaSo, deltaSo (Stand zur Sonne) }
  {          sspb, sspl, sspe (Lage des Satelliten) }
  {Ausgabe:  eklipse (Illumination=0) }
  {Grenzen: 1.1.1987 - 31.12.2000}
var lamSubSoW, sspbbm, dSbm, cA, A, Ac : extended;

begin
  lamSubSoW:=GrtWi+360-alphaSo;
  sspbbm:=sspb*bm;
  dSbm:=deltaSo*bm;
  cA:=sin(dSbm)*sin(sspbbm)+cos(dSbm)*cos(sspbbm)*cos((lamSubSoW+sspl)*bm);
  A:=arccos(cA)/bm;           { Schattenwinkel zur Best. d. Eklipse }
  Ac:=90+arccos(1/sspe)/bm;   { Kritischer Bestrahl.wkl. }
  if A>Ac then eklipse:=true else eklipse:=false
end;

begin
  loaddata;
  if sat=0 then loaddummy;
  s:=paramstr(1);
  if s<>'' then begin
    wahl:=ord(s[1]);
    if (wahl>48) and (wahl<59) then wahl:=wahl-48 else wahl:=0
  end else wahl:=0;
  if (wahl=0) or (wahl>sat) then begin
    writeln;
    writeln('Infos zu SMALL-Sat-Master v',version,':     (C) DB1HZ 1991');
    writeln;
    for j:=1 to sat do writeln('Satellit ',chr(j+48),' ist ',satnam[j]);
    writeln;
    writeln('Syntax: SMALLSAT <Satellit>');
    writeln
  end else begin
    systime;
    berech(wahl);
    j:=stunde+zeitzone;
    if j<0 then begin
      j:=j+24;
      tag:=tag-1;
      if tag=0 then begin
        monat:=monat-1;
        if monat=0 then begin
          monat:=12;
          jahr:=jahr-1
        end;
        tag:=31;
        if (monat=4) or (monat=6) or (monat=9) or (monat=11) then tag:=30;
        if monat=2 then begin
          tag:=28;
          if (jahr=2000) or (jahr and 3>0) then tag:=29
        end
      end
    end;
    if j>23 then begin
      j:=j-24;
      tag:=tag+1;
      if tag>30 then if (monat=4) or (monat=6) or (monat=9) or (monat=11) then tag:=32;
      if (monat=2) and (tag>28) then begin
        if tag=30 then tag:=32;
        if (jahr=2000) or (jahr and 3>0) then tag:=32
      end;
      if tag=32 then begin
        tag:=1;
        monat:=monat+1;
        if monat=13 then begin
          monat:=1;
          jahr:=jahr+1
        end
      end
    end;
    str(minute,v);
    if length(v)=1 then v:='0'+v;
    writeln('SMALL-SAT-Master v',version,'  ',tag,'.',monat,'.',jahr,' ',
            j,':',v,' UTC     (C) DB1HZ 1991');
    writeln;
    writeln('Satellit ',satnam[wahl],' am Amsat-Tag ',amsattag,':');
    writeln;
    writeln('Azimut       = ',azimut:3:1,' Grad');
    writeln('Elevation    = ',elevation:3:1,' Grad');
    writeln('Anomalie     = ',anomalie:3:1,' MA');
    writeln('Orbit        = ',orbit);
    writeln('Entfernung   = ',entfernung,' km');
    writeln('Hhe         = ',sspr,' km');
    write('SSP-Lnge    = ');
    if sspl<0 then writeln(-sspl:3:2,' Grad West')
     else writeln(sspl:3:2,' Grad Ost');
    write('SSP-Breite   = ');
    if sspb<0 then writeln(-sspb:2:2,' Grad Sd')
     else writeln(sspb:2:2,' Grad Nord');
    solephemeriden(0);
    if eklipse then writeln('Illumination = Eklipse')
     else begin
      if (sataa[wahl]<>0) or (satda[wahl]<>0) then begin
        illumination(0);
        write('Illumination = ');
        writeln(round(100*ill),'%');
        writeln('Squint       = ',squint:3:1,' Grad')
      end
    end;
    writeln
  end
end.
